﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;
using MapToolTipSample;
using System.Collections.ObjectModel;

namespace MDIform
{
    public partial class MDIParent1 : Form
    {
        private MapToolTip mapToolTip = new MapToolTip();
        private PointShape currentWorldPoint = new PointShape();
        private Collection<Feature> selectedFeatures = new Collection<Feature>();
        private Label label = new Label();

       public MDIParent1()
        {
            InitializeComponent();

            Form childForm1 = new childForm();
            childForm1.MdiParent = this;
            childForm1.Parent = winformsMap1;

            childForm1.Show();
        }

        private void MDIParent1_Load(object sender, EventArgs e)
        {
            winformsMap1.MapUnit = GeographyUnit.DecimalDegree;
            winformsMap1.CurrentExtent = new RectangleShape(-125, 47, -67, 25);
            winformsMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.FromArgb(255, 198, 255, 255));
            this.winformsMap1.MouseMove += new System.Windows.Forms.MouseEventHandler(this.winformsMap1_MouseMove);

            //Displays the World Map Kit as a background.
            ThinkGeo.MapSuite.DesktopEdition.WorldMapKitWmsDesktopOverlay worldMapKitDesktopOverlay = new ThinkGeo.MapSuite.DesktopEdition.WorldMapKitWmsDesktopOverlay();
            winformsMap1.Overlays.Add(worldMapKitDesktopOverlay);

            InMemoryFeatureLayer inMemoryFeatureLayer = new InMemoryFeatureLayer();
            inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.CreateSimpleCircleStyle(GeoColor.StandardColors.Red, 12, GeoColor.StandardColors.Black, 2);
            inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LayerOverlay layerOverlay = new LayerOverlay();
            layerOverlay.Layers.Add("Point",inMemoryFeatureLayer);
            winformsMap1.Overlays.Add("PointOverlay",layerOverlay);

            label.BackColor = Color.LightYellow;
            label.AutoSize = true;
            label.BorderStyle = BorderStyle.FixedSingle;

            mapToolTip.LayerName = "Point";
            mapToolTip.FieldName = "Tooltip_NAME";
            mapToolTip.StartDelay = 500;
            mapToolTip.DisplayTime = 4000;
            mapToolTip.ToolTipTimer.Tick += new EventHandler(ToolTipTimer_Tick);
            mapToolTip.RemoveToolTipTimer.Tick += new EventHandler(RemoveToolTipTimer_Tick);

            winformsMap1.Refresh();
        }

        void ToolTipTimer_Tick(object sender, EventArgs e)
        {
            DisplayToolTip();
            mapToolTip.ToolTipTimer.Stop();
            mapToolTip.RemoveToolTipTimer.Stop();
            mapToolTip.RemoveToolTipTimer.Start();
        }

        void RemoveToolTipTimer_Tick(object sender, EventArgs e)
        {
            winformsMap1.Controls.Remove(mapToolTip.HoverControl);
            mapToolTip.RemoveToolTipTimer.Stop();
        }

        private void winformsMap1_MouseMove(object sender, MouseEventArgs e)
        {
            //Displays the X and Y in screen coordinates.
            statusStrip1.Items["toolStripStatusLabelScreen"].Text = "X:" + e.X + " Y:" + e.Y;

            //Gets the PointShape in world coordinates from screen coordinates.
            PointShape pointShape = ExtentHelper.ToWorldCoordinate(winformsMap1.CurrentExtent, new ScreenPointF(e.X, e.Y), winformsMap1.Width, winformsMap1.Height);

            //Displays world coordinates.
            statusStrip1.Items["toolStripStatusLabelWorld"].Text = "(world) X:" + Math.Round(pointShape.X, 4) + " Y:" + Math.Round(pointShape.Y, 4);

            if (!mapToolTip.ToolTipTimer.Enabled)
            {
                currentWorldPoint = ExtentHelper.ToWorldCoordinate(winformsMap1.CurrentExtent, e.X, e.Y, winformsMap1.Width, winformsMap1.Height);
                FeatureLayer worldLayer = winformsMap1.FindFeatureLayer(mapToolTip.LayerName);

                double tolerence = 1;
                RectangleShape boundingBox = new RectangleShape(currentWorldPoint.X - tolerence,currentWorldPoint.Y + tolerence,currentWorldPoint.X + tolerence,currentWorldPoint.Y - tolerence);
                worldLayer.Open();
                selectedFeatures = worldLayer.QueryTools.GetFeaturesInsideBoundingBox(boundingBox, new string[1] { mapToolTip.FieldName });
                worldLayer.Close();

                if (selectedFeatures.Count > 0)
                {
                    mapToolTip.ToolTipTimer.Start();
                }
            }
        }

        private void DisplayToolTip()
        {
            ScreenPointF currentScreenPoint = ExtentHelper.ToScreenCoordinate(winformsMap1.CurrentExtent, currentWorldPoint, winformsMap1.Width, winformsMap1.Height);
            if (selectedFeatures.Count > 0)
            {
                if (mapToolTip.HoverControl != null)
                {
                    if (mapToolTip.HoverControl.Text != selectedFeatures[0].ColumnValues[mapToolTip.FieldName].Trim())
                    {
                        label.Text = selectedFeatures[0].ColumnValues[mapToolTip.FieldName].Trim();
                        mapToolTip.HoverControl = label;
                        mapToolTip.HoverControl.Location = new Point((int)(currentScreenPoint.X + mapToolTip.XOffset), (int)(currentScreenPoint.Y + mapToolTip.YOffset));
                        winformsMap1.Controls.Add(mapToolTip.HoverControl);
                    }
                    else
                    {
                        mapToolTip.HoverControl.Hide();
                        mapToolTip.HoverControl.Location = new Point((int)(currentScreenPoint.X + mapToolTip.XOffset), (int)(currentScreenPoint.Y + mapToolTip.YOffset));
                        mapToolTip.HoverControl.Show();
                    }
                }
                else
                {
                    label.Text = selectedFeatures[0].ColumnValues[mapToolTip.FieldName].Trim();
                    mapToolTip.HoverControl = label;
                    mapToolTip.HoverControl.Location = new Point((int)(currentScreenPoint.X + mapToolTip.XOffset), (int)(currentScreenPoint.Y + mapToolTip.YOffset));
                    winformsMap1.Controls.Add(mapToolTip.HoverControl);
                }
            }
        }

        //private void winformsMap1_MouseMove(object sender, MouseEventArgs e)
        //{
            
        //}
     }
}
